"""
helper functions for `goenrich`
"""
import pandas as pd

def generate_background(annotations, df, go_id, entry_id):
    """ generate the backgound from pandas datasets

    >>> O = ontology(...)
    >>> annotations = goenrich.read.gene2go(...)
    >>> background = generate_background(annotations, df, 'GO_ID', 'GeneID')
    >>> propagate(O, background, ...)

    :param annotations: pd.DataFrame containing the annotations
    :param df: pd.DataFrame containing the background genes
    :param go_id: GO id column name
    :param entry_id: Gene id column name
    :returns: dictionary with the background annotations
    """
    return {k : set(v) for k,v in
            pd.merge(annotations, df[[entry_id]]).groupby(go_id)[entry_id]}



import itertools
import networkx as nx

def _tokenize(f):
    token = []
    for line in f:
        if line == '\n':
            yield token
            token = []
        else:
            token.append(line)

def _filter_terms(tokens):
    for token in tokens:
        if token[0] == '[Term]\n':
            yield token[1:]

def _parse_terms(terms):
    for term in terms:
        obsolete = False
        node = {}
        parents = []
        for line in term:
            if line.startswith('id:'):
                id = line[4:-1]
            elif line.startswith('name:'):
                node['name'] = line[6:-1]
            elif line.startswith('namespace:'):
                node['namespace'] = line[11:-1]
            elif line.startswith('is_a:'):
                parents.append(line[6:16])
            elif line.startswith('relationship: part_of'):
                parents.append(line[22:32])
            elif line.startswith('is_obsolete'):
                obsolete = True
                break
        if not obsolete:
            edges = [(p, id) for p in parents] # will reverse edges later
            yield (id, node), edges
        else:
            continue

_filename = 'db/go-basic.obo'

def ontology(filename):
    O = nx.DiGraph()
    with open(filename) as f:
        tokens = _tokenize(f)
        terms = _filter_terms(tokens)
        entries = _parse_terms(terms)
        nodes, edges = zip(*entries)
        O.add_nodes_from(nodes)
        O.add_edges_from(itertools.chain.from_iterable(edges))
        O.graph['roots'] = {data['name'] : n for n, data in O.node.items()
                if data['name'] == data['namespace']}
    for root in O.graph['roots'].values():
        for n, depth in nx.shortest_path_length(O, root).items():
            node = O.node[n]
            node['depth'] = min(depth, node.get('depth', float('inf')))
    O.reverse(copy=False)
    return O



import pandas as pd

path = 'db/webgestalt_example_results/files/final_sig_file_1439064580.tsv'

def _parse(path):
    with open(path) as f:
        res = []
        for line in f:
            if 'GO:' in line:
                res = dict(zip(['namespace', 'name', 'GO_ID'],
                    line.strip().split('\t')))
            elif len(res) > 0:
                res.update(dict(x.split('=') for x in line.strip().split(';')))
                yield res
                res = []

def read(path):
    df = pd.DataFrame(_parse(path))
    df['adjP'] = df['adjP'].astype(float)
    df['rawP'] = df['rawP'].astype(float)
    df['E'] = df['E'].astype(float)
    df['R'] = df['R'].astype(float)
    df['C'] = df['C'].astype(int)
    df['O'] = df['O'].astype(int)
    df['namespace'] = df['namespace'].str.replace(' ', '_')
    return df
